@@ -6,32 +6,30 @@ use warnings;
 use Moose;
 use namespace::autoclean;
 
-use Data::Dumper;
-
-use LWP::UserAgent;
-
-use constant HOSTNAME => 'cgi.search123.uk.com';
-use constant PATH => '/xmlfeed';
-
-our $DEBUG => 0;
-
 use Encode;
+use LWP::UserAgent;
 use URI;
 use XML::LibXML;
+use Time::HiRes qw(gettimeofday);
 
 use WebService::Search123::Ad;
 
+use constant HOSTNAME => 'cgi.search123.uk.com';
+use constant PATH     => '/xmlfeed';
+
+our $DEBUG => 0;
+
 =head1 NAME
 
-WebService::Search123 - Interface to Search123 API.
+WebService::Search123 - Interface to the Search123 API.
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 $VERSION = eval $VERSION;
 
@@ -54,6 +52,27 @@ The Search123 API interface.
 
 Interface to the Search123 platform for searching for ads.
 
+ use WebService::Search123;
+ 
+ $WebService::Search123::DEBUG = 1;
+ 
+ my $s123 = WebService::Search123->new(
+     aid     => 10057,
+     keyword => 'ipod',
+     client  => {
+         ip  => '88.208.204.52',
+         ua  => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8',
+         ref => 'http://www.ultimatejujitsu.com/jujitsu-for-beginners/',
+     },
+ );
+ 
+ binmode STDOUT, ":encoding(UTF-8)";
+ 
+ foreach my $ad ( $s123->ads )
+ {
+     print $ad->title . "\n";
+ }
+
 =cut
 
 =head1 METHODS
@@ -62,15 +81,15 @@ Interface to the Search123 platform for searching for ads.
 
 =head3 ua
 
- The internal LWP::UserAgent to use.
+The internal L<LWP::UserAgent> to use.
 
 =cut
 
-has ua => ( is => 'rw', isa => 'LWP::UserAgent', default => sub { LWP::UserAgent->new } );
+has ua => ( is => 'rw', isa => 'LWP::UserAgent', default => sub { LWP::UserAgent->new( agent => 'WebService-Search123/$VERSION' ) } );
 
 =head3 secure
 
- Flag to indicate whether to use https or http (default).
+Flag to indicate whether to use https or http (default).
 
 =cut
 
@@ -78,7 +97,7 @@ has secure => ( is => 'rw', isa => 'Bool', default => 0 );
 
 =head3 aid
 
- Your account ID with Search123.
+Your account ID with Search123.
 
 =cut
 
@@ -86,7 +105,7 @@ has aid     => ( is => 'rw', isa => 'Num', trigger => \&_reset );
 
 =head3 keyword
 
- The user-supplied keywords to search against.
+The user-supplied keywords to search against.
 
 =cut
 
@@ -94,7 +113,9 @@ has keyword => ( is => 'rw', isa => 'Str', trigger => \&_reset );
 
 =head3 ads
 
- The returned list of ads based on the criteria supplied.
+The returned list of ad objects based on the criteria supplied.
+
+See L<WebService::Search123::Ad> for details on these objects.
 
 =cut
 
@@ -118,15 +139,21 @@ has client => ( is      => 'rw',
                 },
 );
 
+has _type => ( is => 'rw', isa => 'Str', default => 'q', trigger => \&_reset );
 
-
-has type => ( is => 'rw', isa => 'Str', default => 'q', trigger => \&_reset );
-has uid => ( is => 'rw', isa => 'Str', default => 1 );
+has _uid => ( is => 'rw', isa => 'Str', default => 1 );
 
 has _request => ( is => 'rw', isa => 'URI', clearer => '_clear__request' );
 
 has _response => ( is => 'rw', isa => 'HTTP::Response', clearer => '_clear__response' );
 
+=head3 took
+
+How long the underlying HTTP API request took.
+
+=cut
+
+has took => ( is => 'rw', isa => 'Num' );
 
 sub _reset
 {
@@ -146,8 +173,8 @@ sub _build__ads
     $uri->query_form( $uri->query_form, aid => $self->aid );
 
     $uri->query_form( $uri->query_form, query      => $self->keyword           ) if $self->keyword;
-    $uri->query_form( $uri->query_form, type       => $self->type              ) if $self->type;
-    $uri->query_form( $uri->query_form, uid        => $self->uid               ) if $self->uid;
+    $uri->query_form( $uri->query_form, type       => $self->_type             ) if $self->_type;
+    $uri->query_form( $uri->query_form, uid        => $self->_uid              ) if $self->_uid;
     $uri->query_form( $uri->query_form, ip         => $self->get_client('ip')  ) if $self->get_client('ip');
     $uri->query_form( $uri->query_form, client_ref => $self->get_client('ref') ) if $self->get_client('ref');
     $uri->query_form( $uri->query_form, client_ua  => $self->get_client('ua')  ) if $self->get_client('ua');
@@ -156,10 +183,16 @@ sub _build__ads
 
     warn $uri->as_string if $DEBUG;
 
+    my $before = gettimeofday();
+
     $self->_response( $self->ua->get( $uri->as_string ) );
 
+    $self->took( gettimeofday() - $before );
+
     warn $self->_response->code . ' ' . $self->_response->message if $DEBUG;
 
+    warn $self->took if $DEBUG;
+
     warn $self->_response->content if $DEBUG;
 
     my @ads = ();
@@ -172,10 +205,11 @@ sub _build__ads
 
         foreach my $node ( $dom->findnodes('/S123_SEARCH/RETURN/LISTING') )
         {
-            my $ad = WebService::Search123::Ad->new(  title        => $node->findvalue('TITLE'),
-                                                      description  => $node->findvalue('DESCRIPTION'),
-                                                     _redirect_url => $node->findvalue('REDIRECT_URL'),
-                                                      display_url  => $node->findvalue('SITE_URL'),
+            my $ad = WebService::Search123::Ad->new(
+                 title        => $node->findvalue('TITLE'),
+                 description  => $node->findvalue('DESCRIPTION'),
+                _url          => $node->findvalue('REDIRECT_URL'),
+                 display_url  => $node->findvalue('SITE_URL'),
             );
 
             push @ads, $ad;
